Package: RQuantLib
Title: R Interface to the 'QuantLib' Library
Version: 0.4.27
Date: 2026-06-07
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Khanh", "Nguyen", role = "aut", comment = "2009-2010"),
             person("Terry", "Leitch", role = "aut",
                    comment = c(ORCID = "0000-0003-1706-4070", "since 2016")))
Description: The 'RQuantLib' package makes parts of 'QuantLib' accessible from R
 The 'QuantLib' project aims to provide a comprehensive software framework
 for quantitative finance. The goal is to provide a standard open source library
 for quantitative analysis, modeling, trading, and risk management of financial
 assets.
Suggests: tinytest, rgl, shiny
Imports: methods, Rcpp (>= 0.11.0), stats, graphics, zoo
LinkingTo: Rcpp
SystemRequirements: QuantLib library (>= 1.25) from
        https://quantlib.org, Boost library from https://www.boost.org
License: GPL (>= 2)
URL: https://github.com/eddelbuettel/rquantlib,
        https://dirk.eddelbuettel.com/code/rquantlib.html
BugReports: https://github.com/eddelbuettel/rquantlib/issues
RoxygenNote: 6.0.1
LazyLoad: true
NeedsCompilation: yes
Packaged: 2026-06-07 13:24:28 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6419-907X>),
  Khanh Nguyen [aut] (2009-2010),
  Terry Leitch [aut] (ORCID: <https://orcid.org/0000-0003-1706-4070>,
    since 2016)
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2026-06-07 14:10:21 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-07 17:18:17 UTC; unix
Archs: RQuantLib.so.dSYM
