#' Vol Cube Example Data
#'
#' Data for valuing swaption examples including rates and a lognormal vol cube
#'
#' @name vcube
#' @docType data
#' @usage data(vcube)
#' @format two data frames: \code{vcube}, a data frame with four columns: \code{Expiry}, \code{Tenor},
#' \code{LogNormalVol},  and \code{Spread} 
#' @source TBA
NULL

#' Vol Cube Example Data  
#'  
#' Short time series examples
#'  
#' @name tsQuotes 
#' @docType data
#' @format A series of tenors and rates approppriate for calling DiscountCurve
#' @source TBA
NULL
